(**************************************************************************)
(*  Lack - Package manager for non-root users                             *)
(*  (C) 2008 Olivier Schwander <olivier.schwander@ens-lyon.org>           *)
(*  http://chadok.info/lack                                               *)
(*                                                                        *)
(*  This file is part of Lack.                                            *)
(*                                                                        *)
(*  Lack is free software: you can redistribute it and/or modify          *)
(*  it under the terms of the GNU General Public License as published by  *)
(*  the Free Software Foundation, either version 3 of the License, or     *)
(*  (at your option) any later version.                                   *)
(*                                                                        *)
(*  Foobar is distributed in the hope that it will be useful,             *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.       *)
(*                                                                        *)
(**************************************************************************)

open Printf
open Filename

open Package
open Utils

let usage = sprintf "Lack %s, a package manager for all it lacks on your computer" Version.version

let base =
  try Sys.getenv "LACK" with
      Not_found -> (printf "Error: $LACK is not defined\nRun \"eval $(lack-init)\"\n"; exit(1))
let temp = temp_dir ()

let date = chomp (slurp "date +%F")

let install quiet verbose pkg source localfile =
  let name = pkg.pkg_name in
  let version = substitute (total [ ("date", date) ]) pkg.pkg_version in

  let url = pkg.pkg_url in
  let directory = pkg.pkg_directory in
  let fetch_cmd = pkg.pkg_fetch_cmd in
  let extract_cmd = pkg.pkg_extract_cmd in
  let digest = pkg.pkg_digest in

  let patches = pkg.pkg_patches in

  let conf_options = pkg.pkg_conf_options in
  let conf_cmd = pkg.pkg_conf_cmd in

  let build_options = pkg.pkg_build_options in
  let build_cmd = pkg.pkg_build_cmd in

  let install_options = pkg.pkg_install_options in
  let install_cmd = pkg.pkg_install_cmd in

  let file =
    match localfile with
      | None -> basename url
      | Some s -> basename s
  in
  let target = Steps.prepare verbose temp base name version fetch_cmd url in

  let n = 10 in

    Log.task_begin quiet 1 n ("Fetching "^url^"...");
    Log.status quiet (Steps.fetch verbose fetch_cmd url localfile);

    Log.task_begin quiet 2 n "Verifying digest...";
    Log.status quiet (Steps.check verbose target url digest file);

    Log.task_begin quiet 3 n "Extracting...";
    Log.status quiet (Steps.extract verbose extract_cmd directory file);

    Log.task_begin quiet 4 n "Patching...";
    Log.status quiet (Steps.patch verbose patches);

(*     Log.task_begin quiet 5 n "Managing dependencies..."; *)
(*     Log.status quiet (Steps.dependencies verbose ()); *)

    (* We make the difference between configure step and others *)
    Log.task_begin quiet 6 n "Configuring...";
    Log.status quiet (Steps.run ~configure:true verbose conf_cmd conf_options base name version);

    Log.task_begin quiet 7 n "Building...";
    Log.status quiet (Steps.run verbose build_cmd build_options base name version);

    Log.task_begin quiet 8 n "Installing...";
    Log.status quiet (Steps.run verbose install_cmd install_options base name version);

    Log.task_begin quiet 9 n "Registering...";
    Log.status quiet (Steps.register verbose base name version source);

    Log.task_begin quiet 10 n "Cleaning build directory...";
    Log.status quiet (Steps.clean verbose temp);

    ()

let _ =
  let verbose = ref false in
  let quiet = ref false in
  let xml = ref "" in
  let file = ref "" in
  let arguments = (Arg.align [
                     "--verbose", Arg.Set verbose    , " Display output of commands";
                     "--quiet"  , Arg.Set quiet      , " Don't display progression information";
                     "--file"   , Arg.Set_string file, " Don't download the sources, instead, use the file provided";
                   ])
  in
    Arg.parse arguments (fun x -> xml := x) usage;
    let quiet = not !verbose && !quiet in
      if !xml = "" then
        Arg.usage arguments usage
      else
        try
          let pkg = parse !verbose !xml base in
            if !file = "" then
              install quiet !verbose pkg !xml None
            else
              install quiet !verbose pkg !xml (Some !file);
            Sys.chdir base;
            Init.update base
        with
          | Package.File_not_found s -> printf "Unable to find the package file %s\n" s; exit 1
          | _ -> printf "Failure !!! Ouch in %s\n" temp; exit 1

